/*
 * Copyright (c) 2008, 2009, 2010 Marc Balmer <marc@msys.ch>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Author's name may not be used endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
 * IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

/* PostgreSQL utility functions */

#include <stdarg.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>

#include <libpq-fe.h>

PGresult *
PQvexec(PGconn *conn, const char *fmt, ...)
{
	va_list ap;
	PGresult *res;
	int len, ntries;
	char *sql;
	char *errmsg;

	for (ntries = 0; PQstatus(conn) == CONNECTION_BAD && ntries < 10;
	    ntries++) {
		PQreset(conn);
		if (PQstatus(conn) == CONNECTION_BAD)
			sleep(1);
	}
	if (PQstatus(conn) == CONNECTION_BAD) {
		res = PQmakeEmptyPGresult(conn, PGRES_FATAL_ERROR);
		goto finish;
	}

	va_start(ap, fmt);
	len = vasprintf(&sql, fmt, ap);
	va_end(ap);
	if (len == -1) {
		res = PQmakeEmptyPGresult(conn, PGRES_FATAL_ERROR);
		goto finish;
	}
	res = PQexec(conn, sql);
	for (ntries = 0; PQstatus(conn) == CONNECTION_BAD
	    && ntries < 10; ntries++) {
		PQreset(conn);
		if (PQstatus(conn) == CONNECTION_BAD)
			sleep(1);
		else {
			PQclear(res);
			res = PQexec(conn, sql);
		}
	}
	free(sql);
finish:
	return res;
}
